// Copyright: Hiroshi Ichikawa
// Lincense: New BSD Lincense

#include "tx_swig.h"

void Builder::add(const std::string& word) {
  word_list_.push_back(word);
}

void Builder::add_all(const std::vector<std::string>& words) {
  word_list_.insert(word_list_.end(), words.begin(), words.end());
}

void Builder::build(const char* file_name) {
  tx_.build(word_list_, file_name);
  word_list_.clear();
}

bool UnsafeIndex::open(const char* file_name) {
  opened_= tx_.read(file_name) != -1;
  return opened_;
};

int UnsafeIndex::longest_prefix(
    const char* str, int pos, int len, bool match_prefix/*= false*/) {
  if (!opened_) return -1;
  std::size_t plen;
  if (tx_.searchLongestPrefix(str + pos, len, plen, match_prefix)
      != tx_tool::tx::NOTFOUND) {
    return plen;
  } else {
    return -1;
  }
}

bool UnsafeIndex::include(const char* str, int pos, int len) {
  if (!opened_) return false;
  return longest_prefix(str, pos, len) == len;
}

std::vector<std::string> UnsafeIndex::search_prefixes(const char* str, int pos, int len) {
  std::vector<std::string> ret;
  if (!opened_) return ret;
  tx_.searchPrefixes(str + pos, len, ret);
  return ret;
}

std::vector<std::string> UnsafeIndex::search_expansions(
    const char* str, int pos, int len, int limit/*= 0*/) {
  std::vector<std::string> ret;
  if (!opened_) return ret;
  tx_.searchExpansions(str + pos, len, ret, limit);
  return ret;
}
