require "tx_core"


module Tx
    
    class Index
        
        def initialize(file_name)
          @unsafe= UnsafeIndex.new()
          if !@unsafe.open(file_name)
            raise(IOError, "failed to open #{file_name}")
          end
        end
        
        def inspect()
          return "\#<%p:0x%x>" % [self.class, self.object_id]
        end
        
        [:longest_prefix, :include, :search_prefixes, :search_expansions].each() do |name|
          define_method(name) do |*args|
            (str, pos, len, *opt)= args
            raise(ArgumentError, "argument pos is negative") if pos && pos < 0
            pos||= 0
            pos= str.length if pos > str.length
            len= str.length - pos if !len || len < 0 || len > str.length - pos
            @unsafe.__send__(name, str, pos, len, *opt)
          end
        end
        
        def scan(str, &block)
          result= []
          pos= 0
          while pos < str.length
            plen= longest_prefix(str, pos)
            if plen >= 0
              args= [str[pos, plen], pos]
              block ? yield(*args) : result.push(args)
            end
            pos+= plen > 0 ? plen : 1
          end
          return block ? str : result
        end
        
        def gsub(str, &block)
          result= ""
          prev_pos= 0
          scan(str) do |match, pos|
            result << str[prev_pos...pos]
            result << yield(match, pos)
            prev_pos= pos + match.length
          end
          result << str[prev_pos..-1]
          return result
        end
        
    end
    
end
