require "test/unit"
require "tempfile"
require "tx"

class TC_Tx < Test::Unit::TestCase
    
    def setup
      @builder= Tx::Builder.new()
      @builder.add_all(%w(foo ho hog hoga hoge hogeshi))
      @tempfile= Tempfile.new("tx_test")
      @builder.build(@tempfile.path)
      @index= Tx::Index.new(@tempfile.path)
    end

    def test_basic
      assert_equal(4, @index.longest_prefix("hogeshaa"))
      assert_equal(6, @index.longest_prefix("hogeshaa", 0, -1, true))
      assert(@index.include("hoge"))
      assert(!@index.include("hogera"))
      assert_equal(%w(ho hog), @index.search_prefixes("hog"))
      assert_equal(%w(ho hog), @index.search_prefixes("aahog", 2, 5))
      assert_equal(%w(hog hoga hoge hogeshi), @index.search_expansions("hog"))
      assert_equal(%w(hog hoga hoge hogeshi), @index.search_expansions("aahogeshi", 2, 3))
    end
    
    def test_scan
      str= "hohogefugahogaboke"
      expected= [["ho", 0], ["hoge", 2], ["hoga", 10]]
      assert_equal(expected, @index.scan(str))
      result= []
      @index.scan(str) do |s, i|
        result.push([s, i])
      end
      assert_equal(expected, result)
    end
    
    def test_gsub
      result= @index.gsub("hohogefugahogaboke"){ |s, i| s.upcase }
      assert_equal("HOHOGEfugaHOGAboke", result)
    end
    
    def test_open
      assert_raise(IOError) do
        Tx::Index.new("noexist.index")
      end
    end
    
end
